from ehelply_python_sdk.services.service_schemas import *


class BaseNoteMeta(BaseModel):
    original_author: Optional[str]
    author: str
    previous_version: Optional[str]
    next_version: Optional[str]


class GetNote(BaseModel):
    uuid: str
    content: str
    time: str
    meta: BaseNoteMeta


class GetNoteResponse(HTTPResponse):
    uuid: str
    content: str
    time: str
    meta: BaseNoteMeta
    history: List[GetNote] = []


class CreateNote(BaseModel):
    content: str
    time: str
    meta: BaseNoteMeta


class CreateNoteResponse(HTTPResponse):
    uuid: str
    content: str
    time: str
    meta: BaseNoteMeta


class UpdateNote(BaseModel):
    content: str
    time: str
    meta: BaseNoteMeta


class UpdateNoteResponse(HTTPResponse):
    uuid: str
    content: str
    time: str
    meta: BaseNoteMeta


class DeleteNoteResponse(MessageResponse):
    deleted_note_versions: List[str]



from typing import Union
from pydantic import BaseModel

import httpx

from ehelply_python_sdk.utils import SDKConfiguration


class SDKBase:
    """
    Provides the base class for service SDKs
    """
    def __init__(self, sdk_configuration: SDKConfiguration, requests_session: httpx.AsyncClient) -> None:
        self.sdk_configuration: SDKConfiguration = sdk_configuration
        self.requests_session: httpx.AsyncClient = requests_session

    def get_base_url(self) -> str:
        if self.sdk_configuration.base_url_override:
            return self.sdk_configuration.base_url_override
        else:
            return "https://api.prod.ehelply.com"

    def load(self) -> bool:
        return True

    def get_docs_url(self) -> str:
        return "Not available yet"

    def get_service_version(self) -> str:
        return ""



from .schemas import *
from ehelply_python_sdk.services.service_sdk_base import SDKBase


class MetaSDK(SDKBase):
    def get_base_url(self) -> str:
        return super().get_base_url() + "/meta"

    def get_docs_url(self) -> str:
        return super().get_docs_url()

    def get_service_version(self) -> str:
        return super().get_service_version()

    async def create_meta(
            self,
            service: str,
            type_str: str,
            entity_uuid: str,
            meta: CreateMeta
    ) -> Union[GenericHTTPResponse, CreateMetaResponse]:
        if type(service) is not str or type(type_str) is not str or type(entity_uuid) is not str:
            print('Invalid payload when trying to create meta.')
            return ErrorResponse(status_code=0, message="Invalid payload when trying to create meta.")

        data: dict = {
            "meta": meta.dict()
        }

        return transform_response_to_schema(
            await self.requests_session.post(
                self.get_base_url() + "/meta/service/" + service + "/type/" + type_str + "/entity/" + entity_uuid,
                json=data
            ),
            schema=CreateMetaResponse
        )

    async def get_meta(
            self,
            service: str,
            type_str: str,
            entity_uuid: str,
            detailed: bool = False,
            custom: bool = False,
            dates: bool = False
    ) -> Union[GenericHTTPResponse, GetMetaResponse]:
        if type(service) is not str or type(type_str) is not str or type(entity_uuid) is not str or type(
                detailed) is not bool or type(custom) is not bool or type(dates) is not bool:
            print('Invalid payload when trying to get meta.')
            return ErrorResponse(status_code=0, message="Invalid payload when trying to get meta.")

        return transform_response_to_schema(
            await self.requests_session.get(
                self.get_base_url() + "/meta/service/" + service + "/type/" + type_str + "/entity/" + entity_uuid,
                params={"detailed": detailed, "custom": custom, "dates": dates}
            ),
            schema=GetMetaResponse
        )

    async def get_meta_with_id(
            self,
            meta_id: str,
            detailed: bool = False,
            custom: bool = False,
            dates: bool = False
    ) -> Union[GenericHTTPResponse, GetMetaResponse]:
        if type(meta_id) is not str or type(detailed) is not bool or type(custom) is not bool or type(
                dates) is not bool:
            print('Invalid payload when trying to get meta.')
            return ErrorResponse(status_code=0, message="Invalid payload when trying to get meta.")

        return transform_response_to_schema(
            await self.requests_session.get(
                self.get_base_url() + "/meta/" + meta_id,
                params={"detailed": detailed, "custom": custom, "dates": dates}
            ),
            schema=GetMetaResponse
        )

    async def update_meta(
            self,
            service: str,
            type_str: str,
            entity_uuid: str,
            meta: UpdateMeta
    ) -> Union[GenericHTTPResponse, UpdateMetaResponse]:
        if type(service) is not str or type(type_str) is not str or type(entity_uuid) is not str:
            print('Invalid payload when trying to update meta.')
            return ErrorResponse(status_code=0, message="Invalid payload when trying to update meta.")

        data: dict = {
            "meta": meta.dict()
        }

        return transform_response_to_schema(
            await self.requests_session.put(
                self.get_base_url() + "/meta/service/" + service + "/type/" + type_str + "/entity/" + entity_uuid,
                json=data
            ),
            schema=UpdateMetaResponse
        )

    async def update_meta_with_id(
            self,
            meta_id: str,
            meta: UpdateMeta
    ) -> Union[GenericHTTPResponse, UpdateMetaResponse]:
        if type(meta_id) is not str:
            print('Invalid payload when trying to update meta.')
            return ErrorResponse(status_code=0, message="Invalid payload when trying to update meta.")

        data: dict = {
            "meta": meta.dict()
        }

        return transform_response_to_schema(
            await self.requests_session.put(
                self.get_base_url() + "/meta/" + meta_id,
                json=data
            ),
            schema=UpdateMetaResponse
        )

    async def delete_meta(
            self,
            service: str,
            type_str: str,
            entity_uuid: str
    ) -> Union[GenericHTTPResponse, DeleteMetaResponse]:
        if type(service) is not str or type(type_str) is not str or type(entity_uuid) is not str:
            print('Invalid payload when trying to delete meta.')
            return ErrorResponse(status_code=0, message="Invalid payload when trying to delete meta.")

        return transform_response_to_schema(
            await self.requests_session.delete(
                self.get_base_url() + "/meta/service/" + service + "/type/" + type_str + "/entity/" + entity_uuid,
            ),
            schema=DeleteMetaResponse
        )

    async def delete_meta_with_id(
            self,
            meta_id: str
    ) -> Union[GenericHTTPResponse, DeleteMetaResponse]:
        if type(meta_id) is not str:
            print('Invalid payload when trying to delete meta.')
            return ErrorResponse(status_code=0, message="Invalid payload when trying to delete meta.")

        return transform_response_to_schema(
            await self.requests_session.delete(
                self.get_base_url() + "/meta/" + meta_id,
            ),
            schema=DeleteMetaResponse
        )

    async def touch_meta(
            self,
            service: str,
            type_str: str,
            entity_uuid: str
    ) -> Union[GenericHTTPResponse, TouchMetaResponse]:
        if type(service) is not str or type(type_str) is not str or type(entity_uuid) is not str:
            print('Invalid payload when trying to touch meta.')
            return ErrorResponse(status_code=0, message="Invalid payload when trying to touch meta.")

        return transform_response_to_schema(
            await self.requests_session.post(
                self.get_base_url() + "/meta/service/" + service + "/type/" + type_str + "/entity/" + entity_uuid + "/touch",
            ),
            schema=TouchMetaResponse
        )



from typing import Union, Optional

from pydantic import BaseModel
import httpx


class SDKConfiguration(BaseModel):
    access_token: str
    secret_token: str
    project_identifier: str
    partition_identifier: Optional[str] = None  # Required for the AccessSDK in particular
    base_url_override: Optional[str] = None


def make_requests(sdk_configuration: SDKConfiguration) -> httpx.AsyncClient:
    requests_session: httpx.AsyncClient = httpx.AsyncClient(
        headers={
            'X-Access-Token': sdk_configuration.access_token,
            'X-Secret-Token': sdk_configuration.secret_token,
            'Ehelply-Project': sdk_configuration.project_identifier
        },
        timeout=20.0
    )

    return requests_session



from .schemas import *
from ehelply_python_sdk.services.service_sdk_base import SDKBase
from datetime import datetime


class NotesSDK(SDKBase):
    def get_base_url(self) -> str:
        return super().get_base_url() + "/notes"

    def get_docs_url(self) -> str:
        return super().get_docs_url()

    def get_service_version(self) -> str:
        return super().get_service_version()

    async def get_note(
            self,
            note_id: str,
            history: int = 0,
            history_content: bool = True
    ) -> Union[GenericHTTPResponse, GetNoteResponse]:
        return transform_response_to_schema(
            await self.requests_session.get(
                self.get_base_url() + "/notes/" + note_id,
                params={"history": history, "history_content": history_content}
            ),
            schema=GetNoteResponse
        )

    async def delete_note(
            self,
            note_id: str,
            method: str = "previous"
    ) -> Union[GenericHTTPResponse, DeleteNoteResponse]:
        return transform_response_to_schema(
            await self.requests_session.delete(
                self.get_base_url() + "/notes/" + note_id,
                params={"method": method}
            ),
            schema=DeleteNoteResponse
        )

    async def update_note(
            self,
            note_id: str,
            content: str,
            author: str,
    ) -> Union[GenericHTTPResponse, UpdateNoteResponse]:
        if type(content) is not str or type(author) is not str or type(note_id) is not str:
            print('Note entry discarded due to invalid payload.')
            return ErrorResponse(status_code=0, message="Note entry discarded due to invalid payload.")

        data: dict = {
            "note": {
                "content": content,
                "time": datetime.utcnow().isoformat(),
                "meta": {
                    "author": author,
                }
            }
        }

        return transform_response_to_schema(
            await self.requests_session.put(
                self.get_base_url() + "/notes/" + note_id,
                json=data
            ),
            schema=UpdateNoteResponse
        )

    async def create_note(
            self,
            content: str,
            author: str
    ) -> Union[GenericHTTPResponse, CreateNoteResponse]:
        if type(content) is not str or type(author) is not str:
            print('Note entry discarded due to invalid payload.')
            return ErrorResponse(status_code=0, message="Note entry discarded due to invalid payload.")

        data: dict = {
            "note": {
                "content": content,
                "time": datetime.utcnow().isoformat(),
                "meta": {
                    "original_author": author,
                    "author": author,
                }
            }
        }

        return transform_response_to_schema(
            await self.requests_session.post(
                self.get_base_url() + "/notes",
                json=data
            ),
            schema=CreateNoteResponse
        )



from ehelply_python_sdk.services.service_schemas import *


class GetProjectResponse(HTTPResponse):
    uuid: str
    name: str
    created_at: str
    current_spend: int  # Dollar formats represented by a x10000000 integer. Precision to the millonth
    max_spend: int  # Dollar formats represented by a x10000000 integer. Precision to the millonth
    is_spend_maxed: bool
    status: Optional[str] = None


class UpdateProject(BaseModel):
    max_spend: int  # Dollar formats represented by a x10000000 integer. Precision to the millonth


class UpdateProjectResponse(HTTPResponse):
    uuid: str
    name: str
    created_at: str
    current_spend: int  # Dollar formats represented by a x10000000 integer. Precision to the millonth
    max_spend: int  # Dollar formats represented by a x10000000 integer. Precision to the millonth
    is_spend_maxed: bool
    group_m_p: str  # Group UUID that joins members to partition/project
    group_p_c: str  # Group UUID that joins partition/project to eHelply Cloud


class SearchProjectItem(BaseModel):
    uuid: str
    name: str
    created_at: str
    current_spend: int  # Dollar formats represented by a x10000000 integer. Precision to the millonth
    max_spend: int  # Dollar formats represented by a x10000000 integer. Precision to the millonth
    is_spend_maxed: bool


class UsageTypeUnitPrice(BaseModel):
    min_quantity: int  # Quantity formats represented by a x10000000 integer. Precision to the millonth
    max_quantity: int  # Quantity formats represented by a x10000000 integer. Precision to the millonth
    unit_price: int  # Dollar formats represented by a x10000000 integer. Precision to the millonth


class SearchUsageTypeItem(BaseModel):
    key: str
    name: str
    summary: str
    category: str
    service: str
    unit_prices: List[UsageTypeUnitPrice]


class UsageItem(BaseModel):
    uuid: str
    project_uuid: str
    usage_key: str
    year: int
    month: int
    quantity: int  # Quantity formats represented by a x10000000 integer. Precision to the millonth
    estimated_cost: int  # Dollar formats represented by a x10000000 integer. Precision to the millonth
    updated_at: str


class GetUsageResponse(HTTPResponse):
    usage: List[UsageItem]

    def __init__(self, *args, **data: Any) -> None:
        usage: List[UsageItem] = []

        for arg_usage in args:
            usage.append(UsageItem(**arg_usage))

        super().__init__(*args, **data, usage=usage)



from .schemas import *
from ehelply_python_sdk.services.service_sdk_base import SDKBase


class ProductsSDK(SDKBase):
    def get_base_url(self) -> str:
        return super().get_base_url() + "/products"

    def get_docs_url(self) -> str:
        return super().get_docs_url()

    def get_service_version(self) -> str:
        return super().get_service_version()

    async def process_payment(self, project_uuid: str, amount: int) -> Union[GenericHTTPResponse, ProcessPaymentResponse]:
        return transform_response_to_schema(
            await self.requests_session.post(
                self.get_base_url() + "/products/billing/process_payment",
                json={"payment_schema": {
                    "project_uuid": project_uuid,
                    "amount": amount
                }}
            ),
            schema=ProcessPaymentResponse
        )


from ehelply_python_sdk.services.access.sdk import *
from ehelply_python_sdk.services.security.sdk import *
from ehelply_python_sdk.services.notes.sdk import *
from ehelply_python_sdk.services.meta.sdk import *
from ehelply_python_sdk.services.monitor.sdk import *
from ehelply_python_sdk.services.products.sdk import *



from ehelply_python_sdk.services.service_schemas import *


class BaseBasicMeta(BaseModel):
    """
    Basic meta
    """
    name: Optional[str]
    slug: Optional[str]


class BaseDetailedMeta(BaseModel):
    """
    Detailed meta based on Notes
    """
    summary: Optional[str]
    description: Optional[str]
    summary_history: Optional[List[str]] = []
    description_history: Optional[List[str]] = []


class BaseDatesMeta(BaseModel):
    """
    Date based meta
    """
    created_at: Optional[str]
    updated_at: Optional[str]


class GetMetaResponse(HTTPResponse):
    uuid: str
    basic: BaseBasicMeta = BaseBasicMeta()
    detailed: BaseDetailedMeta = BaseDetailedMeta()
    custom: Optional[dict] = {}
    dates: BaseDatesMeta = BaseDatesMeta()


class BaseBasicMetaCreate(BaseModel):
    """
    Basic meta
    """
    name: Optional[str] = None
    slug: bool = True


class BaseDetailedMetaCreate(BaseModel):
    """
    Detailed meta based on Notes
    """
    summary: Optional[str] = None
    description: Optional[str] = None


class CreateMeta(BaseModel):
    basic: Optional[BaseBasicMetaCreate] = BaseBasicMetaCreate()
    detailed: Optional[BaseDetailedMetaCreate] = BaseDetailedMetaCreate()
    custom: Optional[dict] = None


class BaseDetailedMetaReturn(BaseModel):
    """
    Detailed meta based on Notes
    """
    summary_uuid: Optional[str] = None
    description_uuid: Optional[str] = None


class CreateMetaResponse(HTTPResponse):
    uuid: str
    basic: BaseBasicMeta
    detailed: BaseDetailedMetaReturn
    custom: dict = {}
    dates: BaseDatesMeta = BaseDatesMeta()


class UpdateMeta(BaseModel):
    basic: Optional[BaseBasicMetaCreate] = BaseBasicMetaCreate()
    detailed: Optional[BaseDetailedMetaCreate] = BaseDetailedMetaCreate()
    custom: Optional[dict] = None


class UpdateMetaResponse(HTTPResponse):
    uuid: str
    basic: BaseBasicMeta
    detailed: BaseDetailedMetaReturn
    custom: dict = {}
    dates: BaseDatesMeta = BaseDatesMeta()


class DeleteMetaResponse(MessageResponse):
    pass


class TouchMetaResponse(HTTPResponse):
    uuid: str
    basic: BaseBasicMeta
    detailed: BaseDetailedMetaReturn
    custom: dict = {}
    dates: BaseDatesMeta = BaseDatesMeta()



from .schemas import *
from ehelply_python_sdk.services.service_sdk_base import SDKBase


class AuthModel:
    def __init__(
            self,
            access_sdk=None,
            active_participant_uuid=None,
            entity_identifier=None,
            project_uuid=None,
            access_token=None,
            secret_token=None,
            claims=None,
            data=None,
    ) -> None:
        self.access_sdk = access_sdk
        self.active_participant_uuid = active_participant_uuid
        self.entity_identifier = entity_identifier
        self.project_uuid = project_uuid
        self.access_token = access_token
        self.secret_token = secret_token
        self.claims = claims
        self.data = data


class AccessSDK(SDKBase):
    def get_base_url(self) -> str:
        partition_identifier: str = self.sdk_configuration.project_identifier
        if self.sdk_configuration.partition_identifier:
            partition_identifier = self.sdk_configuration.partition_identifier
        return super().get_base_url() + "/access/partitions/" + partition_identifier

    def get_docs_url(self) -> str:
        return super().get_docs_url()

    def get_service_version(self) -> str:
        return super().get_service_version()

    def make_partition_override(self, partition: str) -> str:
        components: List[str] = self.get_base_url().split("/")
        components.pop()
        components.append(partition)
        return "/".join(components)

    async def search_types(self, name: str = None, pagination: Pagination = None) -> Union[GenericHTTPResponse, PageResponse]:
        params: dict = {}
        if name:
            params["name"] = name

        if pagination:
            params["page"] = pagination.next_page
            params["page_size"] = pagination.page_size

        response: PageResponse = transform_response_to_schema(
            await self.requests_session.get(
                self.get_base_url() + "/permissions/types",
                params=params
            ),
            schema=PageResponse
        )

        if is_response_error(response):
            return response

        response.transform_dict(pydantic_type=SearchTypeItem)

        return response

    async def get_type(self, type_uuid: str = None) -> Union[GenericHTTPResponse, GetTypeResponse]:
        return transform_response_to_schema(
            await self.requests_session.get(
                self.get_base_url() + "/permissions/types/" + type_uuid
            ),
            schema=GetTypeResponse
        )

    async def create_type(self, partition_type: CreateType) -> Union[GenericHTTPResponse, CreateTypeResponse]:
        """
        Parameters
        ----------
        partition_type : CreateType
            Pydantic model which defines the parameters of the type to create

        Returns
        -------

        """
        return transform_response_to_schema(
            await self.requests_session.post(
                self.get_base_url() + "/permissions/types",
                json={"partition_type": partition_type.dict()}
            ),
            schema=CreateTypeResponse
        )

    async def create_node(self, partition_type_uuid: str, node: CreateNode) -> Union[GenericHTTPResponse, CreateNodeResponse]:
        return transform_response_to_schema(
            await self.requests_session.post(
                self.get_base_url() + "/permissions/types/" + partition_type_uuid + "/nodes",
                json={"node": node.dict()}
            ),
            schema=CreateNodeResponse
        )

    async def search_nodes(self, type_uuid: str, node: str = None, pagination: Pagination = None) -> Union[
        GenericHTTPResponse, PageResponse]:
        params: dict = {}
        if node:
            params["node"] = node

        if pagination:
            params["page"] = pagination.next_page
            params["page_size"] = pagination.page_size

        response: PageResponse = transform_response_to_schema(
            await self.requests_session.get(
                self.get_base_url() + "/permissions/types/" + type_uuid + "/nodes",
                params=params
            ),
            schema=PageResponse
        )

        if is_response_error(response):
            return response

        response.transform_dict(pydantic_type=SearchNodeItem)

        return response

    async def search_group(self, pagination: Pagination = None) -> Union[GenericHTTPResponse, PageResponse]:
        params: dict = {}

        if pagination:
            params["page"] = pagination.next_page
            params["page_size"] = pagination.page_size

        response: PageResponse = transform_response_to_schema(
            await self.requests_session.get(
                self.get_base_url() + "/who/groups",
                params=params
            ),
            schema=PageResponse
        )

        if is_response_error(response):
            return response

        response.transform_dict(pydantic_type=SearchGroupItem)

        return response

    async def create_group(self, group: CreateGroup) -> Union[GenericHTTPResponse, CreateGroupResponse]:
        return transform_response_to_schema(
            await self.requests_session.post(
                self.get_base_url() + "/who/groups",
                json={"group": group.dict()}
            ),
            schema=CreateGroupResponse
        )

    async def create_role(self, role: CreateRole) -> Union[GenericHTTPResponse, CreateRoleResponse]:
        return transform_response_to_schema(
            await self.requests_session.post(
                self.get_base_url() + "/roles",
                json={"role": role.dict()}
            ),
            schema=CreateRoleResponse
        )

    async def add_node_to_role(self, node_uuid: str, role_uuid: str) -> Union[
        GenericHTTPResponse, MessageResponse]:
        return transform_response_to_schema(
            await self.requests_session.post(
                self.get_base_url() + "/roles/" + role_uuid + "/nodes/" + node_uuid
            ),
            schema=MessageResponse
        )

    async def remove_node_from_role(self, node_uuid: str, role_uuid: str) -> Union[
        GenericHTTPResponse, MessageResponse]:
        return transform_response_to_schema(
            await self.requests_session.delete(
                self.get_base_url() + "/roles/" + role_uuid + "/nodes/" + node_uuid
            ),
            schema=MessageResponse
        )

    async def add_node_to_key(self, node_uuid: str, key_uuid: str) -> Union[
        GenericHTTPResponse, MessageResponse]:
        return transform_response_to_schema(
            await self.requests_session.post(
                self.get_base_url() + "/keys/" + key_uuid + "/nodes/" + node_uuid
            ),
            schema=MessageResponse
        )

    async def remove_node_from_key(self, node_uuid: str, key_uuid: str) -> Union[
        GenericHTTPResponse, MessageResponse]:
        return transform_response_to_schema(
            await self.requests_session.delete(
                self.get_base_url() + "/keys/" + key_uuid + "/nodes/" + node_uuid
            ),
            schema=MessageResponse
        )

    async def add_entity_to_group(self, entity_identifier: str, group_uuid: str) -> Union[
        GenericHTTPResponse, MessageResponse]:
        return transform_response_to_schema(
            await self.requests_session.post(
                self.get_base_url() + "/who/groups/" + group_uuid + "/entities/" + entity_identifier
            ),
            schema=MessageResponse
        )

    async def remove_entity_from_group(self, entity_identifier: str, group_uuid: str) -> Union[
        GenericHTTPResponse, MessageResponse]:
        return transform_response_to_schema(
            await self.requests_session.delete(
                self.get_base_url() + "/who/groups/" + group_uuid + "/entities/" + entity_identifier
            ),
            schema=MessageResponse
        )

    async def add_key_to_entity(
            self,
            entity_identifier: str,
            key_uuid: str
    ) -> Union[GenericHTTPResponse, AddKeyToEntityResponse]:
        return transform_response_to_schema(
            await self.requests_session.post(
                self.get_base_url() + "/who/entities/" + entity_identifier + "/keys/" + key_uuid
            ),
            schema=AddKeyToEntityResponse
        )

    async def remove_key_from_entity(
            self,
            entity_identifier: str,
            key_uuid: str
    ) -> Union[GenericHTTPResponse, RemoveKeyFromEntityResponse]:
        return transform_response_to_schema(
            await self.requests_session.delete(
                self.get_base_url() + "/who/entities/" + entity_identifier + "/keys/" + key_uuid
            ),
            schema=RemoveKeyFromEntityResponse
        )

    async def get_keys_for_entity(
            self,
            entity_identifier: str
    ) -> Union[GenericHTTPResponse, PageResponse]:
        return transform_response_to_schema(
            await self.requests_session.get(
                self.get_base_url() + "/who/entities/" + entity_identifier + "/keys"
            ),
            schema=PageResponse
        )

    async def get_nodes_for_entity(
            self,
            entity_identifier: str
    ) -> Union[GenericHTTPResponse, GetNodesResponse]:
        return transform_response_to_schema(
            await self.requests_session.get(
                self.get_base_url() + "/permissions/nodes/entities/" + entity_identifier
            ),
            schema=GetNodesResponse
        )

    async def get_nodes_for_entity_key(
            self,
            entity_identifier: str,
            key_uuid: str
    ) -> Union[GenericHTTPResponse, GetNodesResponse]:
        return transform_response_to_schema(
            await self.requests_session.get(
                self.get_base_url() + "/permissions/nodes/entities/" + entity_identifier + "/keys/" + key_uuid
            ),
            schema=GetNodesResponse
        )

    async def make_rgt(
            self,
            role_uuid: str,
            group_uuid: str,
            target_identifier: str
    ) -> Union[GenericHTTPResponse, MakeRGTResponse]:
        return transform_response_to_schema(
            await self.requests_session.post(
                self.get_base_url() + "/rgts/roles/" + role_uuid + "/groups/" + group_uuid + "/targets/" + target_identifier,
                json={"limits": []}
            ),
            schema=MakeRGTResponse
        )

    async def automate_role_group_rgt(
            self,
            role: CreateRole,
            group: CreateGroup,
            target_identifier: str,
            entity_identifiers: List[str] = None
    ) -> Union[GenericHTTPResponse, MakeRGTResponse]:

        role_response: CreateRoleResponse = await self.create_role(role=role)

        if is_response_error(role_response):
            return role_response

        group_response: CreateGroupResponse = await self.create_group(group=group)

        if is_response_error(group_response):
            return group_response

        if len(entity_identifiers) > 0 and not group.default:
            for entity_identifier in entity_identifiers:
                await self.add_entity_to_group(entity_identifier=entity_identifier, group_uuid=group_response.uuid)

        return await self.make_rgt(
            role_uuid=role_response.uuid,
            group_uuid=group_response.uuid,
            target_identifier=target_identifier
        )

    async def get_entity_for_key(
            self,
            key_uuid: str,
            partition: str = None
    ) -> Union[GenericHTTPResponse, GetEntityResponse]:

        if partition:
            base_url: str = self.make_partition_override(partition=partition)
        else:
            base_url: str = self.get_base_url()

        return transform_response_to_schema(
            await self.requests_session.get(
                base_url + "/who/entities/keys/" + key_uuid
            ),
            schema=GetEntityResponse
        )

    async def is_entity_allowed(
            self,
            entity_identifier: str,
            target_identifier: str,
            node: str,
            partition: str = None
    ) -> bool:

        if partition:
            base_url: str = self.make_partition_override(partition=partition)
        else:
            base_url: str = self.get_base_url()

        response: Response = await self.requests_session.get(
            base_url + "/auth/targets/" + target_identifier + "/nodes/" + node + "/entities/" + entity_identifier
        )

        if not is_response_error(response) and response.json() is True:
            return True

        # return transform_response_to_schema(response, None)
        return False

    # def is_key_allowed(
    #         self,
    #         access_token: str,
    #         secret_token: str,
    #         target_identifier: str,
    #         node: str,
    #         partition: str = None
    # ) -> bool:
    #     if partition:
    #         base_url: str = self.make_partition_override(partition=partition)
    #     else:
    #         base_url: str = self.get_base_url()
    #
    #     headers: dict = {
    #         "X-Access-Token": access_token,
    #         "X-Secret-Token": secret_token
    #     }
    #
    #     response: Response = await self.requests_session.get(
    #         base_url + "/auth/targets/" + target_identifier + "/nodes/" + node + "/keys",
    #         headers=headers
    #     )
    #
    #     if not is_response_error(response) and response.json() is True:
    #         return True
    #
    #     # return transform_response_to_schema(response, None)
    #     return False

    async def is_allowed(
            self,
            auth_model: AuthModel,
            target_identifier: str,
            node: str,
            partition: str = None
    ) -> bool:
        if auth_model.entity_identifier:
            if await self.is_entity_allowed(
                    entity_identifier=auth_model.entity_identifier,
                    target_identifier=target_identifier,
                    node=node,
                    partition=partition,
            ):
                return True

        # if auth_model.access_token and auth_model.secret_token:
        #     if self.is_key_allowed(
        #             access_token=auth_model.access_token,
        #             secret_token=auth_model.secret_token,
        #             target_identifier=target_identifier,
        #             node=node,
        #             partition=partition,
        #     ):
        #         return True

        return False



from typing import Union, Type, TypeVar, Optional

import httpx

from ehelply_python_sdk.utils import SDKConfiguration, make_requests
from ehelply_python_sdk.services import services
from ehelply_python_sdk.services.service_schemas import is_response_error, ErrorResponse

genericSDKBase = TypeVar('genericSDKBase', bound=services.SDKBase)

CONST_CLIENT_ACCESS: str = "access"
CONST_CLIENT_SECURITY: str = "security"
CONST_CLIENT_NOTES: str = "notes"
CONST_CLIENT_META: str = "meta"
CONST_CLIENT_MONITOR: str = "monitor"
CONST_CLIENT_PRODUCTS: str = "products"


class eHelplySDK:
    """
    eHelply SDK
    """

    def __init__(self, sdk_configuration: SDKConfiguration) -> None:
        self.sdk_configuration: SDKConfiguration = sdk_configuration
        self.requests_session: httpx.AsyncClient = make_requests(sdk_configuration=self.sdk_configuration)

    def _make_client(
            self,
            client: str,
            sdk_configuration: SDKConfiguration = None,
            request_session: httpx.AsyncClient = None
    ) -> genericSDKBase:
        if not sdk_configuration:
            sdk_configuration = self.sdk_configuration

        if not request_session:
            request_session = self.requests_session

        if client == CONST_CLIENT_ACCESS:
            return services.AccessSDK(sdk_configuration=sdk_configuration, requests_session=request_session)

        if client == CONST_CLIENT_SECURITY:
            return services.SecuritySDK(sdk_configuration=sdk_configuration, requests_session=request_session)

        if client == CONST_CLIENT_NOTES:
            return services.NotesSDK(sdk_configuration=sdk_configuration, requests_session=request_session)

        if client == CONST_CLIENT_META:
            return services.MetaSDK(sdk_configuration=sdk_configuration, requests_session=request_session)

        if client == CONST_CLIENT_MONITOR:
            return services.MonitorSDK(sdk_configuration=sdk_configuration, requests_session=request_session)

        if client == CONST_CLIENT_PRODUCTS:
            return services.ProductsSDK(sdk_configuration=sdk_configuration, requests_session=request_session)

    def make_access(
            self,
            sdk_configuration: SDKConfiguration = None,
            request_session: httpx.AsyncClient = None
    ) -> services.AccessSDK:
        return self._make_client(
            client=CONST_CLIENT_ACCESS,
            sdk_configuration=sdk_configuration,
            request_session=request_session
        )

    def make_security(
            self,
            sdk_configuration: SDKConfiguration = None,
            request_session: httpx.AsyncClient = None
    ) -> services.SecuritySDK:
        return self._make_client(
            client=CONST_CLIENT_SECURITY,
            sdk_configuration=sdk_configuration,
            request_session=request_session
        )

    def make_notes(
            self,
            sdk_configuration: SDKConfiguration = None,
            request_session: httpx.AsyncClient = None
    ) -> services.NotesSDK:
        return self._make_client(
            client=CONST_CLIENT_NOTES,
            sdk_configuration=sdk_configuration,
            request_session=request_session
        )

    def make_meta(
            self,
            sdk_configuration: SDKConfiguration = None,
            request_session: httpx.AsyncClient = None
    ) -> services.MetaSDK:
        return self._make_client(
            client=CONST_CLIENT_META,
            sdk_configuration=sdk_configuration,
            request_session=request_session
        )

    def make_monitor(
            self,
            sdk_configuration: SDKConfiguration = None,
            request_session: httpx.AsyncClient = None
    ) -> services.MonitorSDK:
        return self._make_client(
            client=CONST_CLIENT_MONITOR,
            sdk_configuration=sdk_configuration,
            request_session=request_session
        )

    def make_products(
            self,
            sdk_configuration: SDKConfiguration = None,
            request_session: httpx.AsyncClient = None
    ) -> services.ProductsSDK:
        return self._make_client(
            client=CONST_CLIENT_PRODUCTS,
            sdk_configuration=sdk_configuration,
            request_session=request_session
        )



# -*- coding: utf-8 -*-
from setuptools import setup

packages = \
['ehelply_python_sdk',
 'ehelply_python_sdk.services',
 'ehelply_python_sdk.services.access',
 'ehelply_python_sdk.services.meta',
 'ehelply_python_sdk.services.monitor',
 'ehelply_python_sdk.services.notes',
 'ehelply_python_sdk.services.products',
 'ehelply_python_sdk.services.security']

package_data = \
{'': ['*']}

install_requires = \
['cryptography>=2.9.2,<3.0.0',
 'ehelply-logger>=0.0.8,<0.0.9',
 'httpx>=0.18.0,<0.19.0',
 'isodate>=0.6.0,<0.7.0',
 'passlib[bcrypt]>=1.7.2,<2.0.0',
 'pdoc3>=0.9.2,<0.10.0',
 'pydantic>=1.7.3,<2.0.0',
 'pyjwt>=1.7.1,<2.0.0',
 'pyopenssl>=19.1.0,<20.0.0',
 'pytest-asyncio>=0.14.0,<0.15.0',
 'pytest-cov>=2.10.1,<3.0.0',
 'python-jose[cryptography]>=3.1.0,<4.0.0',
 'python-slugify>=4.0.0,<5.0.0',
 'python_dateutil>=2.8.1,<3.0.0',
 'requests>=2.23.0,<3.0.0',
 'typer>=0.2.1,<0.3.0',
 'wheel>=0.34.2,<0.35.0']

entry_points = \
{'console_scripts': ['ehelply_sdk = ehelply_python_sdk.cli:cli_main']}

setup_kwargs = {
    'name': 'ehelply-python-sdk',
    'version': '0.2.7',
    'description': '',
    'long_description': '# SDK',
    'author': 'Shawn Clake',
    'author_email': 'shawn.clake@ehelply.com',
    'maintainer': None,
    'maintainer_email': None,
    'url': 'https://ehelply.com',
    'packages': packages,
    'package_data': package_data,
    'install_requires': install_requires,
    'entry_points': entry_points,
    'python_requires': '>=3.8,<3.9',
}


setup(**setup_kwargs)



from typing import Union

from .schemas import *
from ehelply_python_sdk.services.service_sdk_base import SDKBase


class SecuritySDK(SDKBase):
    def get_base_url(self) -> str:
        return super().get_base_url() + "/security"

    def get_docs_url(self) -> str:
        return super().get_docs_url()

    def get_service_version(self) -> str:
        return super().get_service_version()

    async def create_token(self, length: int = 64) -> Union[GenericHTTPResponse, CreateTokenResponse]:
        response = await self.requests_session.post(
            self.get_base_url() + "/tokens",
            json={"token": {"length": length}}
        )

        if is_response_error(response):
            return transform_response_to_schema(response, None)

        return CreateTokenResponse(
            token=response.json()
        )

    async def create_key(self, key: CreateKey, access_length: int = 48, secret_length: int = 48) -> Union[CreateKeyResponse, GenericHTTPResponse]:
        return transform_response_to_schema(
            await self.requests_session.post(
                self.get_base_url() + "/keys",
                json={"key": key.dict()},
                params={"secret_length": secret_length, "access_length": access_length}
            ),
            schema=CreateKeyResponse
        )

    async def verify_key(self, access: str, secret: str) -> Union[GenericHTTPResponse, VerifyKeyResponse]:
        return transform_response_to_schema(
            await self.requests_session.post(
                self.get_base_url() + "/keys/verify",
                json={
                    "key": {
                        "access": access,
                        "secret": secret
                    }
                }
            ),
            schema=VerifyKeyResponse
        )



from .schemas import *
from ehelply_python_sdk.services.service_sdk_base import SDKBase


class MonitorSDK(SDKBase):
    def get_base_url(self) -> str:
        return super().get_base_url() + "/monitors"

    def get_docs_url(self) -> str:
        return super().get_docs_url()

    def get_service_version(self) -> str:
        return super().get_service_version()

    async def search_projects(
            self,
            name: str = None,
            is_spend_maxed: bool = None,
            pagination: Pagination = None
    ) -> Union[GenericHTTPResponse, PageResponse]:
        params: dict = {}

        if name is not None:
            params["name"] = name

        if is_spend_maxed is not None:
            params["is_spend_maxed"] = is_spend_maxed

        if pagination:
            params["page"] = pagination.next_page
            params["page_size"] = pagination.page_size

        response: PageResponse = transform_response_to_schema(
            await self.requests_session.get(
                self.get_base_url() + "/projects/projects",
                params=params
            ),
            schema=PageResponse
        )

        if is_response_error(response):
            return response

        response.transform_dict(pydantic_type=SearchProjectItem)

        return response

    async def get_project(
            self,
            project_uuid: str
    ) -> Union[GenericHTTPResponse, GetProjectResponse]:
        return transform_response_to_schema(
            await self.requests_session.get(
                self.get_base_url() + "/projects/projects/" + project_uuid
            ),
            schema=GetProjectResponse
        )

    async def get_all_project_usage(
            self,
            project_uuid: str,
            year: int = None,
            month: int = None
    ) -> Union[GenericHTTPResponse, GetUsageResponse]:
        params: dict = {}

        if year is not None:
            params["year"] = year

        if month is not None:
            params["month"] = month

        return transform_response_to_schema(
            await self.requests_session.get(
                self.get_base_url() + "/projects/projects/" + project_uuid + "/usage",
                params=params
            ),
            schema=GetUsageResponse
        )
